# Generated by Django 6.0.2 on 2026-02-24 14:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("hbdjinvoicing", "0005_invoice_customer_vat_id_alter_organisation_vat_id"),
    ]

    operations = [
        migrations.AddField(
            model_name="organisation",
            name="conditions",
            field=models.TextField(blank=True, default=""),
        ),
    ]



from django.db import models
from django.db.models import Sum
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.text import slugify
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _
from django_countries.fields import CountryField
from localflavor.generic.validators import VATINValidator


class BaseModel(models.Model):  # noqa: DJ008
    created_at = models.DateTimeField(
        auto_now_add=True,
        editable=False,
        blank=True,
        verbose_name=_("Date de création"),
    )
    updated_at = models.DateTimeField(
        auto_now=True,
        editable=False,
        blank=True,
        verbose_name=_("Date de modification"),
    )

    class Meta:
        abstract = True


class AddressMixin(models.Model):
    name = models.CharField()
    street = models.TextField()
    zipcode = models.CharField()
    city = models.CharField()
    country = CountryField()

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


def organisation_logo_upload_to(instance, filename):
    return f"organisation/{instance.slug}/logo/{filename}"


class Organisation(AddressMixin):
    email = models.EmailField()
    logo = models.ImageField(upload_to=organisation_logo_upload_to)

    bottom_comment = models.TextField()

    legal_status = models.CharField()
    siren = models.CharField()
    siret = models.CharField()
    vat_id = models.CharField(validators=[VATINValidator])

    conditions = models.TextField(default="", blank=True)

    def __str__(self):
        return self.name

    @property
    def slug(self):
        return slugify(self.name)


def invoice_default_date():
    return now().date()


class Invoice(AddressMixin, BaseModel):
    organisation = models.ForeignKey(
        Organisation, on_delete=models.CASCADE, null=True, default=None
    )
    customer_vat_id = models.CharField(
        default="", blank=True, validators=[VATINValidator]
    )
    number = models.CharField(unique=True)
    date = models.DateField(default=invoice_default_date)
    payment_mode = models.CharField()
    total_vat_excl = models.DecimalField(decimal_places=2, max_digits=9, default=0)
    total_vat_incl = models.DecimalField(decimal_places=2, max_digits=9, default=0)

    credit = models.ForeignKey(
        "Invoice", on_delete=models.CASCADE, null=True, default=None, blank=True
    )

    def __str__(self):
        return f"{self.name} {self.number}"

    def clean(self):
        if not self.id:
            self.total_vat_excl = 0
            self.total_vat_incl = 0
        else:
            self.refresh_totals()

        super().save()

    def refresh_totals(self):
        self.total_vat_excl = sum(
            [line.price_vat_excl for line in self.invoiceline_set.all()]
        )
        self.total_vat_incl = self.total_vat_excl + sum(
            [line.price for line in self.invoicevat_set.all()]
        )

    def refresh_vat(self):
        rows = (
            self.invoiceline_set.values("vat_rate")
            .order_by("vat_rate")
            .annotate(price=Sum("price_vat_excl"))
        )
        for row in rows:
            invoice_vat, created = self.invoicevat_set.get_or_create(
                vat_rate=row["vat_rate"],
                defaults={"price": row["price"] * row["vat_rate"] / 100},
            )
            if not created:
                invoice_vat.price = row["price"] * row["vat_rate"] / 100
                invoice_vat.save()

        self.invoicevat_set.exclude(
            vat_rate__in=[row["vat_rate"] for row in rows]
        ).delete()


class InvoiceVat(BaseModel):
    invoice = models.ForeignKey(Invoice, on_delete=models.CASCADE)
    vat_rate = models.DecimalField(decimal_places=2, max_digits=9)
    price = models.DecimalField(decimal_places=2, max_digits=9)

    def __str__(self):
        return f"{self.invoice} vat {self.vat_rate} %"


class InvoiceLine(BaseModel):
    invoice = models.ForeignKey(Invoice, on_delete=models.CASCADE)
    title = models.CharField()
    descritiption = models.CharField()
    quantity = models.DecimalField(decimal_places=2, max_digits=9)
    unit = models.CharField()
    unit_price = models.DecimalField(decimal_places=2, max_digits=9)
    price_vat_excl = models.DecimalField(decimal_places=2, max_digits=9)
    vat_rate = models.DecimalField(decimal_places=2, max_digits=9)
    price_vat_only = models.DecimalField(decimal_places=2, max_digits=9)
    price_vat_incl = models.DecimalField(decimal_places=2, max_digits=9)

    def __str__(self):
        return f"{self.invoice} line {self.id}"

    def clean(self):
        self.price_vat_excl = self.unit_price * self.quantity
        self.price_vat_only = self.price_vat_excl * self.vat_rate / 100
        self.price_vat_incl = self.price_vat_excl + self.price_vat_only

        super().clean()


@receiver(post_save, sender=InvoiceLine)
def refresh_invoice(sender, instance, **kwargs):
    instance.invoice.refresh_vat()
    instance.invoice.refresh_totals()
    instance.invoice.save()



from django.contrib import admin
from django.urls import path, reverse
from django.utils.safestring import mark_safe
from django.views.generic import DetailView
from django_weasyprint import WeasyTemplateResponseMixin

from .models import Invoice, InvoiceLine, InvoiceVat, Organisation


class InvoiceLineInline(admin.TabularInline):
    model = InvoiceLine
    readonly_fields = ["price_vat_excl", "price_vat_only", "price_vat_incl"]
    extra = 0


class InvoiceVatInline(admin.TabularInline):
    model = InvoiceVat
    readonly_fields = ["vat_rate", "price"]
    can_delete = False
    extra = 0

    def has_add_permission(self, *args):
        return False


class InvoiceHtmlView(DetailView):
    model = Invoice
    template_name = "hbdjinvoicing/pdf.html"


class InvoicePdfView(WeasyTemplateResponseMixin, InvoiceHtmlView):
    pdf_attachment = False

    def get_pdf_filename(self):
        return f"{self.object.name}_{self.object.number}.pdf"


class InvoiceAdmin(admin.ModelAdmin):
    inlines = [InvoiceLineInline, InvoiceVatInline]
    list_display = ["name", "date", "number", "total_vat_incl", "pdf_link"]
    search_fields = ["name", "date", "number", "total_vat_incl"]
    fields = [
        "pdf_link",
        "organisation",
        "number",
        "date",
        "payment_mode",
        "name",
        "street",
        "zipcode",
        "city",
        "country",
        "customer_vat_id",
        "credit",
        "total_vat_excl",
        "total_vat_incl",
    ]
    readonly_fields = [
        "pdf_link",
        "total_vat_excl",
        "total_vat_incl",
    ]

    def pdf_link(self, obj):
        if not obj.pk:
            return ""
        path = reverse("admin:pdf", args=[obj.pk])
        return mark_safe(f'<a href="{path}">PDF</a>')

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            path(
                "<path:pk>/pdf/",
                self.admin_site.admin_view(InvoicePdfView.as_view()),
                name="pdf",
            )
        ]
        return my_urls + urls


class OrganisationAdmin(admin.ModelAdmin):
    pass


admin.site.register(Invoice, InvoiceAdmin)
admin.site.register(Organisation, OrganisationAdmin)



# Generated by Django 5.1.2 on 2025-02-10 14:59

import django.db.models.deletion
import hbdjinvoicing.models
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Customer",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Date de création"
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, verbose_name="Date de modification"
                    ),
                ),
                ("name", models.CharField()),
                ("street", models.CharField()),
                ("zipcode", models.CharField()),
                ("city", models.CharField()),
                ("country", models.CharField()),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Organisation",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField()),
                ("street", models.CharField()),
                ("zipcode", models.CharField()),
                ("city", models.CharField()),
                ("country", models.CharField()),
                ("email", models.EmailField(max_length=254)),
                (
                    "logo",
                    models.ImageField(
                        upload_to=hbdjinvoicing.models.organisation_logo_upload_to
                    ),
                ),
                ("bottom_comment", models.TextField()),
                ("legal_status", models.CharField()),
                ("siren", models.CharField()),
                ("siret", models.CharField()),
                ("vat_id", models.CharField()),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Invoice",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Date de création"
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, verbose_name="Date de modification"
                    ),
                ),
                ("name", models.CharField()),
                ("street", models.CharField()),
                ("zipcode", models.CharField()),
                ("city", models.CharField()),
                ("country", models.CharField()),
                ("number", models.CharField(unique=True)),
                (
                    "date",
                    models.DateField(default=hbdjinvoicing.models.invoice_default_date),
                ),
                ("payment_mode", models.CharField()),
                (
                    "total_vat_excl",
                    models.DecimalField(decimal_places=2, default=0, max_digits=9),
                ),
                (
                    "total_vat_incl",
                    models.DecimalField(decimal_places=2, default=0, max_digits=9),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="hbdjinvoicing.customer",
                    ),
                ),
                (
                    "organisation",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="hbdjinvoicing.organisation",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="InvoiceLine",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Date de création"
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, verbose_name="Date de modification"
                    ),
                ),
                ("title", models.CharField()),
                ("descritiption", models.CharField()),
                ("quantity", models.DecimalField(decimal_places=2, max_digits=9)),
                ("unit", models.CharField()),
                ("unit_price", models.DecimalField(decimal_places=2, max_digits=9)),
                ("price_vat_excl", models.DecimalField(decimal_places=2, max_digits=9)),
                ("vat_rate", models.DecimalField(decimal_places=2, max_digits=9)),
                ("price_vat_only", models.DecimalField(decimal_places=2, max_digits=9)),
                ("price_vat_incl", models.DecimalField(decimal_places=2, max_digits=9)),
                (
                    "invoice",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="hbdjinvoicing.invoice",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="InvoiceVat",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Date de création"
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True, verbose_name="Date de modification"
                    ),
                ),
                ("vat_rate", models.DecimalField(decimal_places=2, max_digits=9)),
                ("price", models.DecimalField(decimal_places=2, max_digits=9)),
                (
                    "invoice",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="hbdjinvoicing.invoice",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



# Generated by Django 5.1.6 on 2025-02-25 18:06

import django_countries.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("hbdjinvoicing", "0002_remove_invoice_customer_delete_customer"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="country",
            field=django_countries.fields.CountryField(max_length=2),
        ),
        migrations.AlterField(
            model_name="organisation",
            name="country",
            field=django_countries.fields.CountryField(max_length=2),
        ),
    ]



# Generated by Django 5.1.6 on 2025-08-22 09:38

import localflavor.generic.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("hbdjinvoicing", "0004_alter_invoice_street_alter_organisation_street"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoice",
            name="customer_vat_id",
            field=models.CharField(
                blank=True,
                default="",
                validators=[localflavor.generic.validators.VATINValidator],
            ),
        ),
        migrations.AlterField(
            model_name="organisation",
            name="vat_id",
            field=models.CharField(
                validators=[localflavor.generic.validators.VATINValidator]
            ),
        ),
    ]



# Generated by Django 5.1.6 on 2026-03-20 17:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("hbdjinvoicing", "0006_organisation_conditions"),
    ]

    operations = [
        migrations.AddField(
            model_name="invoice",
            name="credit",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="hbdjinvoicing.invoice",
            ),
        ),
    ]
